from gi.repository import Adw, Gtk

_xml = """\
<?xml version="1.0" encoding="UTF-8"?>
<interface>
    <requires lib="gtk" version="4.0"/>
    <requires lib="libadwaita" version="1.0"/>
    <template class="YaftiDialog" parent="AdwWindow">
        <property name="title" translatable="yes">Showing Information</property>
        <property name="default-width">500</property>
        <property name="modal">True</property>
        <child>
            <object class="GtkBox">
                <property name="orientation">vertical</property>
                <child>
                    <object class="AdwHeaderBar">
                        <style>
                            <class name="flat"/>
                        </style>
                    </object>
                </child>
                <child>
                  <object class="GtkLabel" id="label_text">
                    <property name="margin-top">10</property>
                    <property name="margin-start">10</property>
                    <property name="margin-end">10</property>
                    <property name="margin-bottom">20</property>
                    <property name="wrap">True</property>
                  </object>
                </child>
            </object>
        </child>
    </template>
</interface>
"""


@Gtk.Template(string=_xml)
class DialogBox(Adw.Window):
    __gtype_name__ = "YaftiDialog"

    def __init__(self, parent=None, **kwargs):
        super().__init__(**kwargs)
        if parent:
            self.set_transient_for(parent)

        sc = Gtk.ShortcutController.new()
        sc.add_shortcut(
            Gtk.Shortcut.new(
                Gtk.ShortcutTrigger.parse_string("Escape"),
                Gtk.CallbackAction.new(lambda x: self.hide()),
            )
        )
        self.add_controller(sc)



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


class NoParentFound(Exception):
    """No parent matched"""


def find_parent(obj, cls=None):
    """Traverse to the parent of a GTK4 component

    Args:
        obj: A GTK4 derived component
        cls: Parent component to find

    Returns:
        The instance of the parent component

    Raises:
        NoParentFound: if a cls is passed and all parents are traversed without a match
    """

    p = obj.get_parent()
    if cls:
        if isinstance(p, cls):
            return p
        if p is None:
            raise NoParentFound(f"no matching parent found for {cls}")

    if p is None:
        return obj

    return find_parent(p, cls)



from pydantic import validate_call


class PackageScreenState:
    __slots__ = ["state"]

    def __new__(cls, id: str):
        if not hasattr(cls, "instances"):
            cls.instances = {}
        if id not in cls.instances:
            cls.instances[id] = super(PackageScreenState, cls).__new__(cls)
        return cls.instances[id]

    def __init__(self, id: str):
        self.state = {}

    @validate_call
    def load(self, data: dict):
        for k, v in data.items():
            self.set(k, v)

    @validate_call
    def remove(self, item: str) -> None:
        del self.state[item]

    @validate_call
    def on(self, item: str) -> None:
        self.set(item, True)

    @validate_call
    def off(self, item: str) -> None:
        self.set(item, False)

    @validate_call
    def toggle(self, item: str) -> bool:
        self.state[item] = not self.state[item]
        return self.get(item)

    @validate_call
    def set(self, item: str, state: bool) -> None:
        self.state[item] = state

    @validate_call
    def get_on(self, prefix: str = "") -> list[str]:
        return [
            item
            for item, value in self.state.items()
            if item.startswith(prefix) and value is True
        ]

    def keys(self) -> list[str]:
        return list(self.state.keys())

    @validate_call
    def get(self, item: str) -> bool:
        return self.state.get(item)



import logging

import gbulb
import gi
from rich.logging import RichHandler

logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[
        RichHandler(
            show_path=False,
            show_time=False,
            rich_tracebacks=True,
            tracebacks_suppress=[gi, logging],
        )
    ],
)

gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")

gbulb.install(gtk=True)



"""
Copyright 2023 Marco Ceppi

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from typing import Any

_listeners = {}


class EventException(Exception):
    """Event system encountered a problem"""


class EventAlreadyRegisteredError(EventException):
    """Event name already exists"""


class EventNotRegisteredError(EventException):
    """Event name does not exist"""


def register(event_name):
    if event_name in _listeners:
        raise EventAlreadyRegisteredError(
            "event is already registered", event=event_name
        )
    _listeners[event_name] = []


def on(event_name: str, fn: Any):
    if event_name not in _listeners:
        raise EventNotRegisteredError("event name not registered", event=event_name)
    if fn in _listeners[event_name]:
        return
    _listeners[event_name].insert(0, fn)


def detach(event_name: str, fn: Any):
    if event_name not in _listeners:
        raise EventNotRegisteredError("event name not registered", event=event_name)
    _listeners[event_name].remove(fn)


async def emit(event_name, *args, **kwargs):
    if event_name not in _listeners:
        raise EventNotRegisteredError("event name not registered", event=event_name)

    for fn in _listeners[event_name]:
        result = await fn(*args, **kwargs)
        if result is not False:
            break
